class ChosenFurniture < ApplicationRecord
  belongs_to :chosen_room, inverse_of: :chosen_furnitures
  belongs_to :furniture, inverse_of: :chosen_furnitures

  has_many :chosen_items, inverse_of: :chosen_furniture, dependent: :destroy
  has_many :furniture_items, through: :chosen_items, inverse_of: :chosen_furnitures

  #has_many :timetables, dependent: :destroy, inverse_of: :chosen_furniture

  validates :furniture, presence: true
  validates :chosen_room, presence: true
  # Whenever I choose a furniture for a room,
  # I must track even the chosen items to add a code to them
  before_save :link_to_items
  before_create :increment_code
  # Items: I need a method to obtain all the items in the
  # current selected Commission
  def records_for_print id
    # TODO: move away from this pseudo code as soon as Bancolini tells me
    # what Mobilart wants
    # chosen_item.name
    # chosen_item.barcode
    # chosen_item.timetables.last.status
    # chosen_item.timetables.last.job_type
    ChosenItem.joins(:chosen_furniture).where(chosen_furnitures: {id: id}).distinct
  end

  def list_associated_items
    true
  end

  amoeba do
    enable
    exclude_association :chosen_items
    exclude_association :furniture_items
  end

  def deck
    chosen_room.chosen_deck.name rescue "-"
  end

  def commission
    chosen_room.chosen_deck.commission.code rescue "-"
  end

  def display_name
    "#{name} presente in #{[commission, deck, chosen_room.name].compact.join(" - ")} contenente #{furniture_items.count} particolari"
  end

  # Necessario per poter cercare nei filed associati
  default_scope {joins(chosen_room: [:room, {chosen_deck: [:deck, :commission ]}])}

  rails_admin do
    navigation_label I18n.t(:operative_section)
    navigation_icon 'fa fa-cubes'
    parent Commission
    weight 7

    edit do
      field :furniture
    end
    list do
      field :commission do
        searchable "commissions.code"
        sortable "commissions.code"
        read_only true
        filterable true
      end
      field :deck do
        searchable "decks.name"
        sortable "decks.name"
        read_only true
        filterable true
      end
      field :chosen_room do
        searchable "rooms.name"
        sortable "rooms.name"
        read_only true
        filterable true
      end
      field :furniture do
        searchable "furnitures.name"
        sortable "furnitures.name"
        filterable true
      end
      field :furniture_items
    end
  end

  def readable_barcode
    "#{chosen_room.readable_barcode}|#{furniture.code}|#{code}"
  end

  def barcode
    "#{chosen_room.barcode}#{furniture.code}#{code}"
  end

  def name
    furniture.name rescue "-"
  end

  private

  def link_to_items
    self.furniture_items = self.furniture.furniture_items
  end

  def increment_code
    # Cerco il numero più alto e incremento
    max_code = ChosenFurniture.order("chosen_furnitures.code DESC").pluck(:code).compact.map(&:to_i).max.to_i
    # Rails.logger.debug "MAX CODE IS:#{max_code} Adding 1: #{(max_code + 1)}"
    self.code = (max_code + 1)
  end
end
