class ChosenItem < ApplicationRecord
  belongs_to :chosen_furniture, inverse_of: :chosen_items #, dependent: :destroy
  belongs_to :furniture_item, inverse_of: :chosen_items
  belongs_to :bundle, inverse_of: :chosen_items

  has_many :timetables, dependent: :destroy, inverse_of: :chosen_item
  has_many :workflows, inverse_of: :chosen_item, dependent: :destroy

  before_create :increment_code

  def self.barcode_concat
    "concat(commissions.code,decks.code,chosen_decks.code,rooms.code,chosen_rooms.code,furnitures.code,chosen_furnitures.code,items.code,LPAD(chosen_items.code::text, 5, '0'))"
  end

  def self.select_with_barcode
    select "chosen_items.*, #{ChosenItem.barcode_concat} as barcode"
  end

  def self.barcode_concat_where barcode
    where ["#{ChosenItem.barcode_concat} = ?", barcode]
  end
  # All the chosen_items must have the full chain of relations up to commission,
  # Otherwise is invalid (and mustn't exist anyway)
  # this gives me the ability to simply query for fields on related tables
  default_scope {
    joins({chosen_furniture: [:furniture, { chosen_room: [:room, {chosen_deck: [:deck, :commission]}]}], furniture_item: :item}).select_with_barcode
  }

  def records_for_print id
    # TODO: move away from this pseudo code as soon as Bancolini tells me
    # what Mobilart wants
    # chosen_item.name
    # chosen_item.barcode
    # chosen_item.timetables.last.status
    # chosen_item.timetables.last.job_type
    ChosenItem.where(chosen_items: {id: id}).distinct
  end

  def room
    chosen_furniture.chosen_room.name rescue "-"
  end

  def deck
    chosen_furniture.chosen_room.chosen_deck.name rescue "-"
  end

  def commission
    chosen_furniture.chosen_room.chosen_deck.commission.code rescue "-"
  end

  def status
    " -overridden by pretty_value- "
  end

  def user
    workflows.last.user.email rescue "-"
  end

  def furniture
    chosen_furniture.furniture.name rescue "-"
  end

  def item
    furniture_item.item.name
  end

  def tiered_times
    tt = self.timetables
    t = (tt.sum(:hours)*60 + tt.sum(:minutes))*60
    t.to_tiered_times true
  end

  def self.filter_by_id
    #Rails.logger.debug "QQQQQQQQQQQQQ: #{params.inspect}"
  end

  rails_admin do
    navigation_label I18n.t(:operative_section)
    parent Commission
    weight 8
    # TODO: mettere a true per i test, mettere a false in production
    visible false

    export do
      field :barcode
      field :commission
      field :deck
      field :room
      field :furniture
      field :item
      field :status
      field :user
    end

    list do
      field :barcode do
        sortable false
        searchable false
        read_only true
        filterable false
      end

      field :commission do
        #column_width 12
        searchable "commissions.code"
        read_only true
        sortable "commissions.code"
        filterable true
      end
      field :deck do
        #column_width 12
        searchable "decks.name"
        sortable "decks.name"
        read_only true
        filterable true
      end
      field :room do
        searchable "rooms.name"
        sortable "rooms.name"
        read_only true
        filterable true
      end
      field :chosen_furniture do
        searchable "furnitures.name"
        sortable "furnitures.name"
        read_only true
        filterable true
      end
      field :furniture_item do
        searchable "items.name"
        sortable "items.name"
        read_only true
        filterable true
      end
      field :status do
        pretty_value do
          obj = bindings[:object]
          wf = obj.workflows.last
          status = []
          status << (wf.stat rescue nil)
          status << (wf.third_party.username rescue nil)
          user = (wf.user.username rescue nil)

          html = []
          html << "<small><strong>#{obj.tiered_times}</strong></small>" unless obj.tiered_times.blank?
          html << "<small>#{status.compact.join("/")}</small>" unless status.compact.blank?
          html << "<small>#{user}</small>" unless user.blank?

          html.join("<br/>").html_safe
        end
      end
      # field :user do
      #   column_width 8
      # end
      # field :times
    end
  end

  def readable_barcode
    "#{chosen_furniture.readable_barcode}|#{furniture_item.item.code}|#{code.rjust(5, '0')}"
  end

  # def barcode
  #   "#{chosen_furniture.barcode}#{furniture_item.item.code}#{code.rjust(5, '0')}"
  # end
  #
  # def barcode=(value)
  # end

  def title
    # "#{furniture_item.item.name rescue nil} (#{readable_barcode rescue nil})"
    furniture_item.item.name rescue "Undefined"
  end

  def name
    "#{chosen_furniture.chosen_room.chosen_deck.commission.name rescue nil}: #{chosen_furniture.name rescue nil} -> #{title}"
  end

  private

  def increment_code
    next_code = (ChosenItem.order("chosen_items.code DESC").pluck(:code).compact.map(&:to_i).max.to_i + 1).to_s
    # Rails.logger.debug "MAX CODE IS:#{max_code} Adding 1: #{(max_code + 1)}"
    # self.barcode = "#{chosen_furniture.barcode}#{furniture_item.item.code}#{next_code.rjust(5, '0')}"
    self.code = next_code
  end
end
