class ChosenRoom < ApplicationRecord
  belongs_to :chosen_deck, inverse_of: :chosen_rooms
  belongs_to :room, inverse_of: :chosen_rooms

  has_many :chosen_furnitures, inverse_of: :chosen_room, dependent: :destroy
  has_many :furnitures, through: :chosen_furnitures, inverse_of: :chosen_rooms
  accepts_nested_attributes_for :chosen_furnitures, allow_destroy: true

  validates :room, presence: true
  validates :chosen_deck, presence: true

  before_create :increment_code
  # Items: I need a method to obtain all the items in the
  # current selected Commission
  def records_for_print id
    # TODO: move away from this pseudo code as soon as Bancolini tells me
    # what Mobilart wants
    # chosen_item.name
    # chosen_item.barcode
    # chosen_item.timetables.last.status
    # chosen_item.timetables.last.job_type
    ChosenItem.joins(chosen_furniture: :chosen_room).where(chosen_rooms: {id: id}).distinct
  end

  def list_associated_items
    true
  end

  def commission
    chosen_deck.commission.code rescue "-"
  end
  def deck
    chosen_deck.deck.name rescue "-"
  end

  def display_name
    "#{room.name} nel ponte #{deck} della commessa #{commission} contenente #{chosen_furnitures.count} mobili"
  end

  default_scope {joins({chosen_deck: [:deck, :commission]})}

  rails_admin do
    navigation_label I18n.t(:operative_section)
    navigation_icon 'fa fa-home'
    parent Commission
    weight 6

    field :commission do
      searchable "commissions.code"
      sortable "commissions.code"
      filterable true
      read_only true
      visible do
        bindings[:controller].params["model_name"] == "chosen_room"
      end
    end
    field :deck do
      searchable "decks.name"
      sortable "decks.name"
      filterable true
      read_only true
      visible do
        bindings[:controller].params["model_name"] == "chosen_room"
      end
    end
    field :room
    list do
      field :furnitures
    end
    show do
      field :chosen_furnitures
    end

    edit do
      # Differenziare fra quando sono nella configurazione stanze o ponti
      field :room do
        visible do
          bindings[:controller].params["model_name"] == "chosen_deck"
        end
      end
      field :chosen_furnitures do
        visible do
          bindings[:controller].params["model_name"] == "chosen_room"
        end
      end
    end
  end

  amoeba do
    enable
    include_association :chosen_furnitures
  end

  def readable_barcode
    "#{chosen_deck.readable_barcode rescue "" }|#{room.code}|#{code}"
  end

  def barcode
    "#{chosen_deck.barcode rescue "" }#{room.code}#{code}"
  end

  def name
    # "#{room.name rescue nil} (#{readable_barcode rescue nil})"
    room.name rescue "Undefined"
  end

  private

  def increment_code
    max_code = ChosenRoom.order("chosen_rooms.code DESC").pluck(:code).compact.map(&:to_i).max.to_i
    # Rails.logger.debug "MAX CODE IS:#{max_code} Adding 1: #{(max_code + 1)}"
    self.code = (max_code + 1)
  end
end
