class AddBarcodeToChosenItem < ActiveRecord::Migration[4.2]
  class ChosenItem < ActiveRecord::Base
    belongs_to :chosen_furniture, inverse_of: :chosen_items #, dependent: :destroy
    belongs_to :furniture_item, inverse_of: :chosen_items
    belongs_to :bundle, inverse_of: :chosen_items

    has_many :timetables, dependent: :destroy, inverse_of: :chosen_item
    has_many :workflows, inverse_of: :chosen_item, dependent: :destroy

    before_create :increment_code

    default_scope {joins({chosen_furniture: [:furniture, { chosen_room: [:room, {chosen_deck: [:deck, :commission]}]}], furniture_item: :item})}

    def records_for_print id
      # TODO: move away from this pseudo code as soon as Bancolini tells me
      # what Mobilart wants
      # chosen_item.name
      # chosen_item.barcode
      # chosen_item.timetables.last.status
      # chosen_item.timetables.last.job_type
      ChosenItem.where(chosen_items: {id: id}).distinct
    end

    def room
      chosen_furniture.chosen_room.name rescue "-"
    end

    def deck
      chosen_furniture.chosen_room.chosen_deck.name rescue "-"
    end

    def commission
      chosen_furniture.chosen_room.chosen_deck.commission.code rescue "-"
    end

    def status
      workflows.last.stat rescue "Non in lavorazione"
    end

    def user
      workflows.last.user.email rescue "-"
    end

    def furniture
      chosen_furniture.furniture.name rescue "-"
    end

    def item
      furniture_item.item.name
    end

    def times
      t = (timetables.sum(:hours)*60 + timetables.sum(:minutes))*60000
      mm, ss = t.divmod(60)            #=> [4515, 21]
      hh, mm = mm.divmod(60)           #=> [75, 15]
      dd, hh = hh.divmod(24)           #=> [3, 3]
      "#{dd} giorni, #{hh} ore, #{mm} minuti"
    end

    def self.filter_by_id
      Rails.logger.debug "QQQQQQQQQQQQQ: #{params.inspect}"
    end

    rails_admin do
      navigation_label I18n.t(:operative_section)
      parent Commission
      weight 8
      # TODO: mettere a true per i test, mettere a false in production
      visible false

      export do
        field :commission
        field :deck
        field :room
        field :furniture
        field :item
        field :status
        field :user
        field :times
      end

      list do
        field :commission do
          searchable "commissions.code"
          read_only true
          sortable "commissions.code"
          filterable true
        end
        field :deck do
          searchable "decks.name"
          sortable "decks.name"
          read_only true
          filterable true
        end
        field :room do
          searchable "rooms.name"
          sortable "rooms.name"
          read_only true
          filterable true
        end
        field :chosen_furniture do
          searchable "furnitures.name"
          sortable "furnitures.name"
          read_only true
          filterable true
        end
        field :furniture_item do
          searchable "items.name"
          sortable "items.name"
          read_only true
          filterable true
        end
        field :status
        field :user
        field :times
      end
    end

    def readable_barcode
      "#{chosen_furniture.readable_barcode}|#{furniture_item.item.code}|#{code.rjust(5, '0')}"
    end

    def title
      # "#{furniture_item.item.name rescue nil} (#{readable_barcode rescue nil})"
      furniture_item.item.name rescue "Undefined"
    end

    def name
      "#{chosen_furniture.chosen_room.chosen_deck.commission.name rescue nil}: #{chosen_furniture.name rescue nil} -> #{title}"
    end

    private

    def increment_code
      next_code = (ChosenItem.order("chosen_items.code DESC").pluck(:code).compact.map(&:to_i).max.to_i + 1)
      # Rails.logger.debug "MAX CODE IS:#{max_code} Adding 1: #{(max_code + 1)}"
      self.code = next_code
      self.barcode = "#{chosen_furniture.barcode}#{furniture_item.item.code}#{next_code.rjust(5, '0')}"
    end
  end

  def change
    add_column :chosen_items, :barcode, :string
    add_index :chosen_items, :barcode

    # After adding the field, I must resave all the Chosen Items to make the calculate the right barcode
    ChosenItem.all.each do |ci|
      puts "before save - id: #{ci.id}, code: #{ci.code}, barcode: #{ci.barcode}"
      ci.barcode = "#{ci.chosen_furniture.barcode}#{ci.furniture_item.item.code}#{ci.code.rjust(5, '0')}"
      ci.save
      puts "after save - id: #{ci.id}, code: #{ci.code}, barcode: #{ci.barcode}"
    end
  end
end
